
#include <stdio.h>
#include <avr/io.h>
#define SYSCLK 16000000UL			// Prozessortakt fr delay festlegen
#include <util/delay.h>

//--- Definition der Steuer- und Datenbits
#define CONTROLPORT PORTB
#define CONTROLDIRPORT DDRB
#define CTRL_RS PORT3
#define CTRL_SS PORT4
#define CTRL_MOSI PORT5
#define CTRL_MISO PORT6
#define CTRL_SCK PORT7

//*********************************************************************************
//*** Beginn Subfunktionen ********************************************************
	
//*********************************************************************************

void ctoggle_RS (void)	// RS nach Control-Befehl toggeln
	{
		CONTROLPORT |= _BV(CTRL_RS);		// CTRL_RS HIGH bringen
	    CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
	}
	
//*********************************************************************************

void wtoggle_RS (void)	// RS nach Schreib-Befehl toggeln
	{
	    CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		CONTROLPORT |= _BV(CTRL_RS);		// CTRL_RS HIGH bringen
	}

//*********************************************************************************

void cursor_off (void)	// Cursor ausschalten
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00001100;
		_delay_ms(0.1);
		ctoggle_RS ();
	}
	
//*********************************************************************************

void cursor_underline (void)	// Cursor Unterstrich
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00001110;
		_delay_ms(0.1);
		ctoggle_RS ();
	}
	
//*********************************************************************************

void cursor_blink (void)	// Cursor blinkend
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00001101;
		_delay_ms(0.2);
		ctoggle_RS ();
	}
	
//*********************************************************************************	

void set_cursor (unsigned char pos)	// Cursor an Position setzen
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 128+pos;				//  0..15 -> Zeile 1
									// 16..31 -> Zeile 2
		_delay_ms(0.2);				// 32..47 -> Zeile 3
		ctoggle_RS ();
	}
	
//*********************************************************************************	

void clear_home (void)	// Display lschen, Cursor an Position home setzen
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00000001;	
		_delay_ms(1.2);
		ctoggle_RS ();
	}
	
//*********************************************************************************	

void clear_lastpos (void)	// Display lschen, Cursor an letzte Position setzen
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00000010;	
		_delay_ms(1.2);
		ctoggle_RS ();
	}

//*********************************************************************************	

void set_contrast (unsigned char con)	// Kontrast einstellen (0..15)
	{
		CONTROLPORT &= ~_BV(CTRL_RS);		// CTRL_RS Low bringen
		SPDR = 0b00111001;	// Instruction-table 1
		_delay_ms(0.1);
		ctoggle_RS ();
		con = con & 0x0f;			// nur die unteren 4 bit verwenden
		SPDR = 0b01110000+con;	// Kontrast setzen
		_delay_ms(0.1);
		ctoggle_RS ();
		SPDR = 0b00111000;	// zurck zu Instruction-table 0
		_delay_ms(0.1);
		ctoggle_RS ();
	}
	
//*********************************************************************************	

int write_char (char car, FILE *unused)	// Byte an Cursorposition ausgeben 
	{
		CONTROLPORT |= _BV(CTRL_RS);		// CTRL_RS HIGH bringen
		SPDR = car;
		_delay_ms(.1);
		wtoggle_RS ();
		return 0;
	}

//*********************************************************************************
	
void display_init (void)	// Display initialisieren
	{
	  _delay_ms(25);					// Warten bis Display sicher versorgt
	  _delay_ms(25);
	  CONTROLPORT &= ~_BV(CTRL_RS);	// CTRL_RS Low bringen
	  SPDR = 0b00111001;	// Datenlnge, Zeilenzahl, doppelte Hhe, Instruction-table 1
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b00010101;	// Bias, Fixed-on-high (3-Line) and fixed on low (other apps)
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b01010000;	// Icon-Display on/off, Booster on/off, Contrast set (internal-follower-mode)
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b01101100;	// Spannungsfolger und Verstrkung setzen
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b01111101;	// Kontrast setzen
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b00111000;	// zurck zu Instruction-table 0
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b00001100;	// Display, Cursorposition (Unterstrich), Cursor blinken
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b00000001;	// Display lschen, Cursor home
	  _delay_ms(2);
	  ctoggle_RS ();
	  SPDR = 0b00000110;	// Cursor auto-increment
	  _delay_ms(2);
	  ctoggle_RS ();
	}

/**************************************************************************************************/

void SPI_Init (void)
{
	CONTROLDIRPORT |= _BV(CTRL_RS)|_BV(CTRL_MOSI)|_BV(CTRL_SCK)|_BV(CTRL_SS);
	SPCR |= (0 << SPIE) | /* SPI Interrupt disable */
           (1 << SPE)  | /* SPI Enable */
           (0 << DORD) | /* Data Order: MSB first */
           (1 << MSTR) | /* Master mode */
           (0 << CPOL) | /* Clock Polarity: CTRL_SCK low when idle */
           (0 << CPHA) | /* Clock Phase: sample on rising CTRL_SCK edge */
           (0 << SPR1) | /* Clock Frequency: f_OSC / 4  => 4Mbit */
           (0 << SPR0);
    SPSR &= ~(1 << SPI2X); /* No doubled clock frequency */
	_delay_us(20);
}

//*************************************************************************************
//*************************************************************************************

// Output-Stream VOR main zuweisen:
// FILE lcd_str = FDEV_SETUP_STREAM(write_char, NULL, _FDEV_SETUP_WRITE);
//
// stderr fr fprintf IN main zuweisen:
// stderr = &lcd_str;
//
// Aufruf/Beschreiben des Displays z.B. mit:
//
// clear_home();
// set_cursor(0);
// fprintf(stderr, "Displaytest");
//
// Cursorpositionen der drei Zeilen siehe Funktion 'set_cursor'
 
 
 
 
 


